# BUILD INCLUDE build/template.inc
# Authors: Louise M. Lever
#          Llion Evans
# --------------------------------------------------------------------------------
# Cygwin is a large collection of GNU and Open Source tools which provide 
# functionality similar to a Linux distribution on Windows.
# https://www.cygwin.com/
#
# ParaFEM can be compiled to run in parallel locally on a Windows machine using 
# a combination of Cygwin and MPI. It is recommended to select the following
# MPI packages (and their dependencies) whilst installing Cygwin; libopenmpi, 
# libopenmpi-devel, openmpi, libopenmpicxx1, libopenmpifh2, libopenmpiuse1.
# Additionally, for code development packages related to ssh, svn and the gcc
# fortran compiler should be included.
#
# The installation of MPI can be checked by downloading a hello_world fortran
# program from https://svn.open-mpi.org/source/raw/ompi_1.8/examples/hello_mpifh.f
# This can be compiled and run in Cygwin by issuing the following commands:
# $ mpifort -o hello_mpifh hello_mpifh.f
# $ mpirun -np 2 ./hello_mpifh
# --------------------------------------------------------------------------------

# --------------------------------------------------------------------------------
# Include file for Cygwin
# --------------------------------------------------------------------------------

# --------------------------------------------------------------------------------
# Generic settings for compilation
# --------------------------------------------------------------------------------
FC=mpifort
INSTALL_CMD=mv

# --------------------------------------------------------------------------------
# FLAGS for EXTERNAL LIBRARY compliation
# --------------------------------------------------------------------------------
EXT_LIB_ARPACK_FC=gcc
EXT_LIB_ARPACK_FFLAGS=-fdefault-real-8 -fdefault-double-8
EXT_LIB_ARPACK_LDFLAGS=

EXT_LIB_METIS_CC=
EXT_LIB_METIS_OPTFLAGS=
EXT_LIB_METIS_COPTIONS=
EXT_LIB_METIS_LDOPTIONS=
EXT_LIB_METIS_AR=
EXT_LIB_METIS_RANLIB=

EXT_LIB_GAF77_FC=gcc
EXT_LIB_GAF77_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
EXT_LIB_GAF77_LDFLAGS=

# --------------------------------------------------------------------------------
# FLAGS for MODULE/LIBRARY compliation
# --------------------------------------------------------------------------------
# Form is LIB_(MPI|SERIAL|OPENMP|GPU)_(REL|DBG)_(FFLAGS|LDFLAGS)
# These flags are directly used by the module makefiles
# --------------------------------------------------------------------------------
LIB_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
LIB_MPI_DBG_FFLAGS=-g
  
LIB_SERIAL_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
LIB_SERIAL_DBG_FFLAGS=-g -fdefault-real-8 -fdefault-double-8
  
LIB_BEM_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
LIB_BEM_DBG_FFLAGS=-g
  
LIB_BEM_P_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
LIB_BEM_P_DBG_FFLAGS=-g

# --------------------------------------------------------------------------------
# DEFAULT FLAGS for all program groups
# --------------------------------------------------------------------------------
# Form is DEFAULT_(P12|XX|PROG8|GAITFEM)_(MPI|SERIAL|OPENMP|GPU)_(REL|DBG)_(FFLAGS|LDFLAGS)
#   where applicable.
# These are NOT directly used in any makefiles; specific program flags are assigned
# these defaults below and may be overridden.
# --------------------------------------------------------------------------------

# generic P12 group FFLAGS and LDFLAGS
DEFAULT_P12_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_P12_MPI_DBG_FFLAGS=-g
DEFAULT_P12_MPI_REL_LDFLAGS=
DEFAULT_P12_MPI_DBG_LDFLAGS=

# generic 5ED group FFLAGS and LDFLAGS
DEFAULT_5ED_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_5ED_MPI_DBG_FFLAGS=-g 
DEFAULT_5ED_MPI_REL_LDFLAGS=
DEFAULT_5ED_MPI_DBG_LDFLAGS=

# generic XX group FFLAGS and LDFLAGS
DEFAULT_XX_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_XX_MPI_DBG_FFLAGS=-g
DEFAULT_XX_MPI_REL_LDFLAGS=
DEFAULT_XX_MPI_DBG_LDFLAGS=

# generic PROG8 group FFLAGS and LDFLAGS
DEFAULT_PROG8_BEM_REL_FFLAGS=
DEFAULT_PROG8_BEM_DBG_FFLAGS=
DEFAULT_PROG8_BEM_REL_LDFLAGS=
DEFAULT_PROG8_BEM_DBG_LDFLAGS=
DEFAULT_PROG8_BEM_P_REL_FFLAGS=
DEFAULT_PROG8_BEM_P_DBG_FFLAGS=
DEFAULT_PROG8_BEM_P_REL_LDFLAGS=
DEFAULT_PROG8_BEM_P_DBG_LDFLAGS=

# generic GAITFEM group FFLAGS and LDFLAGS
DEFAULT_GAITFEM_SERIAL_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_GAITFEM_SERIAL_DBG_FFLAGS=-g
DEFAULT_GAITFEM_SERIAL_REL_LDFLAGS=
DEFAULT_GAITFEM_SERIAL_DBG_LDFLAGS=

# generic RFEM group FFLAGS and LDFLAGS
DEFAULT_RFEM_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_RFEM_MPI_DBG_FFLAGS=-g 
DEFAULT_RFEM_MPI_REL_LDFLAGS=
DEFAULT_RFEM_MPI_DBG_LDFLAGS=

# --------------------------------------------------------------------------------
# DEFAULT FLAGS for all tool groups
# --------------------------------------------------------------------------------
# Form is DEFAULT_(PREPROC|POSTPROC)_(FC|CC|MPI|SERIAL|OPENMP|GPU)_(REL|DBG)_(FFLAGS|LDFLAGS)
#   where applicable.
# These are NOT directly used in any makefiles; specific tool flags are assigned
# these defaults below and may be overridden.
# --------------------------------------------------------------------------------

# generic PREPROC group FFLAGS and LDFLAGS
DEFAULT_PREPROC_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_PREPROC_MPI_DBG_FFLAGS=-O2
DEFAULT_PREPROC_MPI_REL_LDFLAGS=
DEFAULT_PREPROC_MPI_DBG_LDFLAGS=

DEFAULT_PREPROC_CC_REL_FFLAGS=-O2
DEFAULT_PREPROC_CC_DBG_FFLAGS=-g
DEFAULT_PREPROC_FC_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_PREPROC_FC_DBG_FFLAGS=-g

# generic POSTPROC group FFLAGS and LDFLAGS
DEFAULT_POSTPROC_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
DEFAULT_POSTPROC_MPI_DBG_FFLAGS=-g
DEFAULT_POSTPROC_MPI_REL_LDFLAGS=
DEFAULT_POSTPROC_MPI_DBG_LDFLAGS=

# --------------------------------------------------------------------------------
# --------------------------------------------------------------------------------
# SPECIFIC FLAGS for PROGRAMS and TOOLS
# By default these inherit the defaults defined above.
# NOTE: If a specific program needs different flags then modify below.
# NOTE: It may be useful to create a new include file for a modified architecture.
#   e.g., <arch>_test.inc and use MACHINE=<arch>_test in the build script.
# --------------------------------------------------------------------------------
# --------------------------------------------------------------------------------

# --------------------------------------------------------------------------------
# PROGRAM FLAGS for all programs.
# --------------------------------------------------------------------------------
# All use their program group defaults defined above unless overridden below
# --------------------------------------------------------------------------------
P121_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P121_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P121_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P121_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P123_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P123_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P123_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P123_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P125_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P125_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P125_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P125_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P126_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P126_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P126_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P126_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)
  
P129_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P129_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P129_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P129_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)
  
P1210_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P1210_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P1210_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P1210_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)
  
PROG81_BEM_REL_FFLAGS=$(DEFAULT_PROG8_BEM_REL_FFLAGS)
PROG81_BEM_REL_LDFLAGS=$(DEFAULT_PROG8_BEM_REL_LDFLAGS)
PROG81_BEM_DBG_FFLAGS=$(DEFAULT_PROG8_BEM_DBG_FFLAGS)
PROG81_BEM_DBG_LDFLAGS=$(DEFAULT_PROG8_BEM_DBG_LDFLAGS)
  
PROG82_BEM_REL_FFLAGS=$(DEFAULT_PROG8_BEM_REL_FFLAGS)
PROG82_BEM_REL_LDFLAGS=$(DEFAULT_PROG8_BEM_REL_LDFLAGS)
PROG82_BEM_DBG_FFLAGS=$(DEFAULT_PROG8_BEM_DBG_FFLAGS)
PROG82_BEM_DBG_LDFLAGS=$(DEFAULT_PROG8_BEM_DBG_LDFLAGS)
  
PROG83_BEM_P_REL_FFLAGS=$(DEFAULT_PROG8_BEM_P_REL_FFLAGS)
PROG83_BEM_P_REL_LDFLAGS=$(DEFAULT_PROG8_BEM_P_REL_LDFLAGS)
PROG83_BEM_P_DBG_FFLAGS=$(DEFAULT_PROG8_BEM_P_DBG_FFLAGS)
PROG83_BEM_P_DBG_LDFLAGS=$(DEFAULT_PROG8_BEM_P_DBG_LDFLAGS)
  
GAITFEM_SERIAL_REL_FFLAGS=$(DEFAULT_GAITFEM_SERIAL_REL_FFLAGS)
GAITFEM_SERIAL_REL_LDFLAGS=$(DEFAULT_GAITFEM_SERIAL_REL_LDFLAGS)
GAITFEM_SERIAL_DBG_FFLAGS=$(DEFAULT_GAITFEM_SERIAL_DBG_FFLAGS)
GAITFEM_SERIAL_DBG_LDFLAGS=$(DEFAULT_GAITFEM_SERIAL_DBG_LDFLAGS)

RFEMSOLVE_MPI_REL_FFLAGS=$(DEFAULT_RFEM_MPI_REL_FFLAGS)
RFEMSOLVE_MPI_REL_LDFLAGS=$(DEFAULT_RFEM_MPI_REL_LDFLAGS)
RFEMSOLVE_MPI_DBG_FFLAGS=$(DEFAULT_RFEM_MPI_DBG_FFLAGS)
RFEMSOLVE_MPI_DBG_LDFLAGS=$(DEFAULT_RFEM_MPI_DBG_LDFLAGS)

P121_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P121_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P121_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P121_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P122_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P122_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P122_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P122_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P123_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P123_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P123_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P123_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P124_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P124_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P124_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P124_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P125_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P125_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P125_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P125_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P126_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P126_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P126_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P126_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P127_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P127_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P127_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P127_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P128_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P128_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P128_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P128_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P129_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P129_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P129_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P129_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P1210_5_MPI_REL_FFLAGS=$(DEFAULT_P12_MPI_REL_FFLAGS)
P1210_5_MPI_REL_LDFLAGS=$(DEFAULT_P12_MPI_REL_LDFLAGS)
P1210_5_MPI_DBG_FFLAGS=$(DEFAULT_P12_MPI_DBG_FFLAGS)
P1210_5_MPI_DBG_LDFLAGS=$(DEFAULT_P12_MPI_DBG_LDFLAGS)

P1211_5_MPI_REL_FFLAGS=$(DEFAULT_5ED_MPI_REL_FFLAGS)
P1211_5_MPI_REL_LDFLAGS=$(DEFAULT_5ED_MPI_REL_LDFLAGS)
P1211_5_MPI_DBG_FFLAGS=$(DEFAULT_5ED_MPI_DBG_FFLAGS)
P1211_5_MPI_DBG_LDFLAGS=$(DEFAULT_5ED_MPI_DBG_LDFLAGS)

XX1_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX1_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX1_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX1_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX2_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX2_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX2_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX2_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX3_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX3_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX3_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX3_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX4_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX4_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX4_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX4_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX5_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX5_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX5_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX5_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX6_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX6_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX6_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX6_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX7_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX7_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX7_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX7_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX8_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX8_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX8_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX8_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX9_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX9_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX9_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX9_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX10_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX10_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX10_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX10_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX11_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX11_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX11_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX11_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX12_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX12_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX12_MPI_DBG_FFLAGS=-g
XX12_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX12_b_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX12_b_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX12_b_MPI_DBG_FFLAGS=-g
XX12_b_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

XX13_MPI_REL_FFLAGS=$(DEFAULT_XX_MPI_REL_FFLAGS)
XX13_MPI_REL_LDFLAGS=$(DEFAULT_XX_MPI_REL_LDFLAGS)
XX13_MPI_DBG_FFLAGS=$(DEFAULT_XX_MPI_DBG_FFLAGS)
XX13_MPI_DBG_LDFLAGS=$(DEFAULT_XX_MPI_DBG_LDFLAGS)

# --------------------------------------------------------------------------------
# TOOL FLAGS for selected tools.
# --------------------------------------------------------------------------------
# All use their tool group defaults defined above unless overridden below
# --------------------------------------------------------------------------------

# Pre Processing tools
# Note: There are no flags for INP2D
MG2D_MPI_REL_FFLAGS=$(DEFAULT_PREPROC_MPI_REL_FFLAGS)
MG2D_MPI_REL_LDFLAGS=$(DEFAULT_PREPROC_MPI_REL_LDFLAGS)
MG2D_MPI_DBG_FFLAGS=$(DEFAULT_PREPROC_MPI_DBG_FFLAGS)
MG2D_MPI_DBG_LDFLAGS=$(DEFAULT_PREPROC_MPI_DBG_LDFLAGS)

P12MESHGEN_MPI_REL_FFLAGS=$(DEFAULT_PREPROC_MPI_REL_FFLAGS)
P12MESHGEN_MPI_REL_LDFLAGS=$(DEFAULT_PREPROC_MPI_REL_LDFLAGS)
P12MESHGEN_MPI_DBG_FFLAGS=$(DEFAULT_PREPROC_MPI_DBG_FFLAGS)
P12MESHGEN_MPI_DBG_LDFLAGS=$(DEFAULT_PREPROC_MPI_DBG_LDFLAGS)

PARTITIONER_REL_FFLAGS=$(DEFAULT_PREPROC_CC_REL_FFLAGS)
PARTITIONER_DBG_FFLAGS=$(DEFAULT_PREPROC_CC_DBG_FFLAGS)

DINOMESH_REL_FFLAGS=$(DEFAULT_PREPROC_FC_REL_FFLAGS)
DINOMESH_DBG_FFLAGS=$(DEFAULT_PREPROC_FC_DBG_FFLAGS)
  
FEARFY_REL_FFLAGS=$(DEFAULT_PREPROC_FC_REL_FFLAGS)
FEARFY_DBG_FFLAGS=$(DEFAULT_PREPROC_FC_DBG_FFLAGS)

RFEMFIELD_MPI_REL_FFLAGS=$(DEFAULT_PREPROC_MPI_REL_FFLAGS)
RFEMFIELD_MPI_REL_LDFLAGS=$(DEFAULT_PREPROC_MPI_REL_LDFLAGS)
RFEMFIELD_MPI_DBG_FFLAGS=$(DEFAULT_PREPROC_MPI_DBG_FFLAGS)
RFEFIELDM_MPI_DBG_LDFLAGS=$(DEFAULT_PREPROC_MPI_DBG_LDFLAGS)

RFEMBC_MPI_REL_FFLAGS=$(DEFAULT_PREPROC_MPI_REL_FFLAGS)
RFEMBC_MPI_REL_LDFLAGS=$(DEFAULT_PREPROC_MPI_REL_LDFLAGS)
RFEMBC_MPI_DBG_FFLAGS=$(DEFAULT_PREPROC_MPI_DBG_FFLAGS)
RFEMBC_MPI_DBG_LDFLAGS=$(DEFAULT_PREPROC_MPI_DBG_LDFLAGS)

# Post Processing tools
BND2DIS_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
BND2DIS_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
BND2DIS_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
BND2DIS_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

D2OFF_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
D2OFF_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
D2OFF_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
D2OFF_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

MAT2NUMPE_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
MAT2NUMPE_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
MAT2NUMPE_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
MAT2NUMPE_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

UPVW2DIS_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
UPVW2DIS_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
UPVW2DIS_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
UPVW2DIS_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

TTRB2TTR_MPI_REL_FFLAGS=-fdefault-real-8 -fdefault-double-8 -O2
TTRB2TTR_MPI_REL_LDFLAGS=-g
TTRB2TTR_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
TTRB2TTR_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

PF2ENSIBIN_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
PF2ENSIBIN_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
PF2ENSIBIN_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
PF2ENSIBIN_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

PF2ENSI_CASE_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
PF2ENSI_CASE_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
PF2ENSI_CASE_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
PF2ENSI_CASE_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

ENSIBIN2ASCII_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
ENSIBIN2ASCII_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
ENSIBIN2ASCII_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
ENSIBIN2ASCII_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)

TTR_ENSIBIN2ASCII_MPI_REL_FFLAGS=$(DEFAULT_POSTPROC_MPI_REL_FFLAGS)
TTR_ENSIBIN2ASCII_MPI_REL_LDFLAGS=$(DEFAULT_POSTPROC_MPI_REL_LDFLAGS)
TTR_ENSIBIN2ASCII_MPI_DBG_FFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_FFLAGS)
TTR_ENSIBIN2ASCII_MPI_DBG_LDFLAGS=$(DEFAULT_POSTPROC_MPI_DBG_LDFLAGS)