PROGRAM pf2ensi_case
!------------------------------------------------------------------------------
!      Program pf2ensi_case    Tool to produce ensight gold formatted case file
!                              Under development, currently only for xx12 using 
!                              4-node tetrahedra
!------------------------------------------------------------------------------
  
  USE, INTRINSIC :: ISO_C_BINDING ! to output C binary file
  
  IMPLICIT NONE
  
!------------------------------------------------------------------------------
! 1. Declare variables used in the main program
!------------------------------------------------------------------------------
  
  INTEGER,PARAMETER   :: iwp=SELECTED_REAL_KIND(15)
  INTEGER, PARAMETER  :: ndim=3,nodof=1,nprops=5
  INTEGER             :: meshgen,partitioner,np_types,nels,nn,nr,nip,nod
  INTEGER             :: loaded_nodes,fixed_freedoms,nstep,npri,npri_chk,ntime
  INTEGER             :: limit,el_print,i_o,nreal,n_int,npp,j_loc,j_step
  INTEGER             :: i,j,k,prnwidth,err
  REAL(iwp),PARAMETER :: zero = 0.0_iwp
  REAL(iwp)           :: val0,dtim,theta,tol,etype,real_time
  CHARACTER(LEN=50)   :: fname,job_name
  CHARACTER(LEN=15)   :: element,chk
  CHARACTER(LEN=80)   :: cbuffer
  CHARACTER(LEN=10)   :: keyword
  
!------------------------------------------------------------------------------
! 2. Declare dynamic arrays
!------------------------------------------------------------------------------
  
  REAL(iwp),ALLOCATABLE :: temp_real(:),prop(:,:),val_f(:),val(:,:)
  REAL(iwp),ALLOCATABLE :: timesteps_real(:,:)
  INTEGER,ALLOCATABLE   :: temp_int(:),node(:),sense(:),timesteps_int(:,:)
  
!------------------------------------------------------------------------------
! 3. Read job_name from the command line.
!    Read control data
!------------------------------------------------------------------------------
  
  CALL GETARG(1,job_name)
  
  PRINT *, "Starting conversion of ParaFEM input files"

  fname = job_name(1:INDEX(job_name, " ") -1) // ".dat"
  OPEN(10,FILE=fname,STATUS='OLD',ACTION='READ')
  READ(10,*) element,meshgen,partitioner,np_types,                            &
             nels,nn,nr,nip,nod,loaded_nodes,fixed_freedoms,                  &
             chk,val0,                                                        &
             ntime,theta,tol,limit,                                           &
             el_print,i_o
  CLOSE(10)
  
  fname = job_name(1:INDEX(job_name, " ") -1) // ".time"
  OPEN(21,FILE=fname, STATUS='OLD', ACTION='READ')
  READ(21,*) keyword, ntime, nreal, n_int
  !Read the material labels line
  READ(21,*)                  ! skip line
  
  ALLOCATE(timesteps_real(ntime,nreal),timesteps_int(ntime,n_int))
  
  DO i = 1,ntime
    READ(21,*)j, timesteps_real(i,:), timesteps_int(i,:)
  END DO
  
  CLOSE(21)
  
  npp = 0
  DO i=1,ntime
    npp = npp + timesteps_int(i,1)/timesteps_int(i,2)
  END DO
  
!----------------------------------------------------------------------------
! 4. Open files
!----------------------------------------------------------------------------
  
!------------------------------------------------------------------------------
! 9. Write case file
!------------------------------------------------------------------------------
  
  PRINT *, "Writing case file"
  fname   = job_name(1:INDEX(job_name, " ")-1)//".bin.ensi.case"
  OPEN(24,FILE=fname)

  WRITE(24,'(A/A)')    "#", "# Post-processing file generated by subroutine &
                             &WRITE_ENSI in "
  WRITE(24,'(A,A,/A)') "#"," Smith, Griffiths and Margetts, 'Programming the &
                             &Finite Element Method',","# Wiley, 2013."        
  WRITE(24,'(A/A/A)')  "#","# Ensight Gold Format","#"
  WRITE(24,'(2A/A)')   "# Problem name: ",job_name(1:INDEX(job_name, " ")-1),"#"
  WRITE(24,'(A/A/A)')  "FORMAT","type:  ensight gold","GEOMETRY"
  WRITE(24,'(2A/A)')   "model: 1  ",job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.geo',"VARIABLE"
  WRITE(24,'(2A)')     "scalar per element:  material      ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.MATID'
  WRITE(24,'(2A)')     "scalar per element:  material_kx   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.ELMAT_1_kx'
  WRITE(24,'(2A)')     "scalar per element:  material_ky   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.ELMAT_2_ky'
  WRITE(24,'(2A)')     "scalar per element:  material_kz   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.ELMAT_3_kz'
  WRITE(24,'(2A)')     "scalar per element:  material_rho  ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.ELMAT_4_rho'
  WRITE(24,'(2A)')     "scalar per element:  material_cp   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.ELMAT_5_cp'
  WRITE(24,'(2A)')     "scalar per node: 1   temperature   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.NDTTR-******'
  WRITE(24,'(2A)')     "scalar per node:     fixed_nodes   ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.NDFIX'
  WRITE(24,'(2A)')     "scalar per node:     loaded_nodes  ",                &
                        job_name(1:INDEX(job_name, " ")-1)//'.bin.ensi.NDLDS'
  WRITE(24,'(A/A)')     "TIME","time set:     1"
  WRITE(24,'(A,I5)')    "number of steps:",npp+1
  WRITE(24,'(A,I5)')    "filename start number:",1
  WRITE(24,'(A,I5)')    "filename increment:",1
  WRITE(24,'(A)')       "time values:"
  
  prnwidth = 10
  j=1
!  PRINT *, "Starting Time Steps"
  WRITE(24,'(E12.5)',ADVANCE='no') zero
  DO j_step=1,ntime
    dtim     = timesteps_real(j_step,1)
    nstep    = timesteps_int(j_step,1)
    npri     = timesteps_int(j_step,2)
    npri_chk = timesteps_int(j_step,3)

!    PRINT *, "ntime = ",j_step
    timesteps: DO j_loc=1,nstep

      real_time = 0
      IF(j_step>1)THEN
        DO i = 1,j_step-1
          real_time = real_time + timesteps_real(i,1)*timesteps_int(i,1)
        END DO
      END IF

!      real_time = i*dtim
      real_time = real_time + j_loc*dtim
      IF(j_loc/npri*npri==j_loc)THEN
!        PRINT *, "real_time = ",real_time
        WRITE(24,'(E12.5)',ADVANCE='no') real_time
        j=j+1
      END IF
      IF(j==prnwidth)THEN
          WRITE(24,*)''
          j=0
      END IF
    END DO timesteps
  END DO
  WRITE(24,*)''
  CLOSE(24)
  
  PRINT *, "ENSIGHT GOLD case file complete"
  
! CALL shutdown() - not needed unless the program uses MPI
  
END PROGRAM pf2ensi_case
